package com.cedarsoft.hudson.bears;

import org.apache.commons.io.IOUtils;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;

import java.io.IOException;
import java.net.MalformedURLException;

/**
 *
 */
public class BearUpdater {
  @NonNls
  @NotNull
  private final StatusUrls statusUrls;
  @NotNull
  private final PlugConnector plugConnector;

  private volatile boolean shuttingDown;

  public BearUpdater() throws MalformedURLException {
    statusUrls = new StatusUrls( "http://build.cedarsoft.de:8080/" );
    plugConnector = new PlugConnector( "192.168.0.109" );

    System.out.println( "Starting Bear Updater..." );
  }

  public static void main( String[] args ) throws Exception {
    new BearUpdater().run();
  }

  private void run() throws IOException {
    Runtime.getRuntime().addShutdownHook( new Thread( new Runnable() {
      public void run() {
        shuttingDown = true;
        System.out.println( "Disabling all" );
        try {
          plugConnector.disableAll();
        } catch ( IOException e ) {
          e.printStackTrace();
        }
      }
    } ) );

    while ( true ) {
      if ( Thread.currentThread().isInterrupted() ) {
        return;
      }

      ProjectsState projectsState = parseProjectsState();
      if ( shuttingDown ) {
        return;
      }
      switchBears( projectsState );

      try {
        Thread.sleep( 1000 * 60 );
      } catch ( InterruptedException ignore ) {
      }
    }
  }

  private void switchBears( @NotNull ProjectsState criticalState ) throws IOException {
    PlugState[] plugStates = new PlugState[]{PlugState.OFF, PlugState.OFF, PlugState.OFF, PlugState.OFF};

    if ( criticalState.hasBuildingProject() ) {
      plugStates[0] = PlugState.ON;
    }

    if ( criticalState.hasFailedProject() ) {
      plugStates[3] = PlugState.ON;
    } else {
      if ( criticalState.hasInstableProject() ) {
        plugStates[2] = PlugState.ON;
      } else {
        if ( criticalState.hasStableProject() ) {
          plugStates[1] = PlugState.ON;
        }
      }
    }
    plugConnector.switchPlugs( plugStates );
  }

  @NotNull
  private ProjectsState parseProjectsState() throws IOException {
    String statusXml = IOUtils.toString( statusUrls.getAll().openStream() );

    ProjectsState projectsState = new ProjectsState();

    for ( ProjectState state : ProjectState.values() ) {
      if ( statusXml.contains( state.getHudsonColor() + "</color>" ) ) {
        projectsState.enable( state );
      }
    }
    return projectsState;
  }
}
