package com.cedarsoft.hudson.bears;

import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

/**
 *
 */
public class PlugConnector {
  @NotNull
  @NonNls
  private final String server;
  @NotNull
  private final ReadWriteLock lock = new ReentrantReadWriteLock();


  public PlugConnector( @NotNull @NonNls String server ) {
    this.server = server;
  }

  /**
   * Switches the plugs
   *
   * @param states the states (first state switches the first plug)
   * @throws IOException
   */
  public void switchPlugs( @NotNull PlugState[] states ) throws IOException {
    for ( int i = 0; i < states.length; i++ ) {
      switchPlug( i, states[i] );
    }
  }

  public void disableAll() throws IOException {
    for ( int i = 0; i < 8; i++ ) {
      switchPlug( i, PlugState.OFF );
    }
  }

  public void switchPlug( int plugIndex, @NotNull PlugState state ) throws IOException {
    switchPlug( plugIndex, state == PlugState.ON );
  }

  public void switchPlug( int plugIndex, boolean enabled ) throws IOException {
    createUrl( plugIndex, enabled ).openStream().close();
  }

  @NotNull
  private URL createUrl( int plugIndex, boolean enabled ) throws MalformedURLException {
    int type = enabled ? 0 : 1;
    return new URL( "http", server, "ctrl.cgi?F" + plugIndex + "=" + type );
  }
}
